/*
 * Copyright (C) 2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "include/liblogger/register_results.hh"

#include "include/liblogger/exceptions.hh"

#include "src/util.hh"

#include "libpg/query.hh"
#include "libpg/sql_state/code.hh"

namespace Fred {
namespace LibLogger {

namespace {

void insert_result(
        const LibPg::PgRwTransaction& _rw_transaction,
        const Result& _result,
        const LogEntry::ServiceId& _service_id)
{
    const auto query = LibPg::make_query() <<
            // clang-format off
            "INSERT INTO result_code ("
               "result_code, "
               "name, "
               "service_id) "
            "VALUES (" <<
                LibPg::parameter<LogEntry::ResultCode>().as_integer() << ", " <<
                LibPg::parameter<LogEntry::ResultName>().as_text() << ", " <<
                LibPg::parameter<LogEntry::ServiceId>().as_integer() << ") "
                "ON CONFLICT DO NOTHING";
            // clang-format on

    try
    {
        LibPg::PgResultNoData{
            exec(_rw_transaction, query, {_result.code, _result.name, _service_id})};
    }
    catch (const LibPg::ExecFailure& e)
    {
        if (e.error_code() == LibPg::SqlState::string_data_right_truncation)
        {
            // the only string that can be truncated from text to varchar(64) is _result.name
            throw InvalidResultName{};
        }
        throw;
    }
}

} // namespace Fred::LibLogger::{anonymous}

void register_results(
        const LibPg::PgRwTransaction& _rw_transaction,
        const LogEntry::ServiceName& _service,
        const std::vector<Result>& _results)
{
    const auto service_id = get_service_id_by_name(_rw_transaction, _service);

    for (const auto& result : _results)
    {
        insert_result(_rw_transaction, result, service_id);
    }
}

} // namespace Fred::LibLogger
} // namespace Fred
